"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var _ = require("underscore");
var index_1 = require("../../../index");
var formCore_1 = require("../formCore");
var validationWrapper_1 = require("../validationWrapper");
function calcTabIndex(tabIndex, fieldIndex) {
    if (tabIndex === undefined || tabIndex === -1) {
        return tabIndex;
    }
    return tabIndex + fieldIndex;
}
exports.CodeInput = function (props) {
    var onChange = props.onChange, validationMode = props.validationMode, lengthPerBox = props.lengthPerBox, numeric = props.numeric, type = props.type, className = props.className, tabIndex = props.tabIndex, value = props.value, placeholder = props.placeholder, readonly = props.readonly;
    var _a = React.useState(null), focusIndex = _a[0], setFocusIndex = _a[1];
    var uniq = React.useRef(Math.random());
    var storedKey = React.useRef(null);
    var buildValue = React.useCallback(function () {
        var value = "";
        var current = document.getElementById("input_" + uniq.current + "_0");
        lengthPerBox.forEach(function () {
            value += current.value;
            current = current.nextSibling;
        });
        if (numeric) {
            value = parseInt(value, 10);
        }
        if (onChange) {
            onChange(value);
        }
    }, [uniq, lengthPerBox, onChange, numeric]);
    var focusNext = React.useCallback(function (e) {
        var movingBack = false;
        var current = e.target;
        var el;
        var currentVal = current.value.trim();
        var lpb = lengthPerBox[focusIndex];
        var retFromStore = false;
        currentVal = currentVal.slice(0, lpb);
        if (e.keyCode === 8) {
            movingBack = true;
            if (currentVal.length === 0) {
                el = current.previousSibling;
            }
            else {
                el = current;
            }
        }
        else {
            el = current.nextSibling;
            if (storedKey.current && el && !el.value) {
                el.value = storedKey.current;
                storedKey.current = null;
                retFromStore = true;
            }
        }
        if (currentVal.length < lpb && !movingBack) {
            return;
        }
        if (el) {
            el.focus();
            if (el && el.value && !movingBack && !retFromStore) {
                el.value = "";
            }
        }
        buildValue();
    }, [lengthPerBox, focusIndex, storedKey, buildValue]);
    var handlePaste = React.useCallback(function (e) {
        var length = _.reduce(lengthPerBox, function (memo, num) { return memo + num; }, 0);
        var pasted = e.clipboardData.getData("text/plain").replace(/\s/g, "");
        pasted = pasted.substr(0, length);
        if (numeric) {
            var parsed = parseInt(pasted, 10);
            if (isNaN(parsed)) {
                e.preventDefault();
                return false;
            }
        }
        var current = e.target;
        var splitArray = [];
        var currentIndex = 0;
        lengthPerBox.forEach(function (lpb) {
            var chunk = pasted.substr(currentIndex, lpb);
            currentIndex += lpb;
            splitArray.push(chunk);
            current.value = chunk;
            current = current.nextSibling;
        });
        buildValue();
    }, [lengthPerBox, buildValue, numeric]);
    var handleFocus = React.useCallback(function (index) { return function () {
        setFocusIndex(index);
        storedKey.current = null;
    }; }, [storedKey]);
    var keyDown = React.useCallback(function (e) {
        // console.log(e.key)
        var lpb = lengthPerBox[focusIndex];
        var selectionLength = window.getSelection().toString().length;
        if (selectionLength === lpb) {
            return;
        }
        if (e.currentTarget.value.length === lpb) {
            if (e.keyCode >= 48 && e.keyCode <= 57) {
                storedKey.current = e.key;
            }
            if (e.keyCode >= 65 && e.keyCode <= 90) {
                storedKey.current = e.key;
            }
        }
    }, [lengthPerBox, focusIndex, storedKey]);
    var handleClick = React.useCallback(function (e) {
        if (e.currentTarget.value) {
            e.currentTarget.select();
        }
    }, []);
    React.useEffect(function () {
        if (value) {
            var currentIndex_1 = 0;
            lengthPerBox.forEach(function (lpb, i) {
                var input = document.getElementById("input_" + uniq.current + "_" + i);
                var chunk = value.substr(currentIndex_1, lpb);
                currentIndex_1 += lpb;
                input.value = chunk;
            });
        }
    }, []);
    var validationMessage = formCore_1.DataValidationMessage.get(props);
    var classes = React.useMemo(function () { return index_1.ClassHelpers.classNames("armstrong-input", "code-input", className, {
        "show-validation": validationMode !== "none" && validationMessage,
    }); }, [className, validationMode, validationMessage]);
    return (React.createElement("div", { className: classes },
        React.createElement("div", null, lengthPerBox.map(function (lpb, i) { return (React.createElement("input", { id: "input_" + uniq.current + "_" + i, className: "code-input-field", tabIndex: calcTabIndex(tabIndex, i), key: "cb_" + uniq.current + "_" + i, type: type || "text", placeholder: placeholder, maxLength: lpb, readOnly: readonly, onClick: handleClick, onFocus: handleFocus(i), onKeyUp: focusNext, onKeyDown: keyDown, onPaste: handlePaste, onChange: buildValue })); })),
        React.createElement(validationWrapper_1.ValidationLabel, { message: validationMessage, mode: validationMode })));
};
exports.CodeInput.defaultProps = {
    lengthPerBox: [2, 2, 2],
    validationMode: "none",
};
