"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var classHelpers_1 = require("../../../utilities/classHelpers");
var dateHelpers_1 = require("../../../utilities/dateHelpers");
var formatting_1 = require("../../../utilities/formatting");
var hooks_1 = require("../../../utilities/hooks");
var grid_1 = require("../../layout/grid");
var formCore_1 = require("../formCore");
var formHooks_1 = require("../formHooks");
var validationWrapper_1 = require("../validationWrapper");
var options_1 = require("./options");
exports.DateInput = function (props) {
    var dayLabel = props.dayLabel, monthLabel = props.monthLabel, yearLabel = props.yearLabel, futureDates = props.futureDates, yearsFromNow = props.yearsFromNow, startYearCap = props.startYearCap, className = props.className, validationMode = props.validationMode, disabled = props.disabled, datePartOrder = props.datePartOrder, tabIndex = props.tabIndex, date = props.date, onChange = props.onChange;
    var _a = React.useState({ day: null, month: null, year: null, date: null }), dateState = _a[0], setDateState = _a[1];
    var daysArrayByMonth = React.useMemo(function () {
        return dateHelpers_1.DateHelpers.getDaysArrayByMonth(dateState.month, dateState.year);
    }, [dateState.month, dateState.year]);
    React.useEffect(function () {
        validateProps();
        if (date) {
            setDateState(dateHelpers_1.DateHelpers.getDateParts(date));
        }
    }, []);
    var validateProps = React.useCallback(function () {
        if (datePartOrder.indexOf("month") === -1) {
            // tslint:disable-next-line:no-console
            console.error("A DateInput must include `month` in the datePartOrder");
        }
    }, [datePartOrder]);
    hooks_1.useDidUpdateEffect(function () {
        validateProps();
        if (date) {
            setDateState(dateHelpers_1.DateHelpers.getDateParts(date, true));
        }
        else {
            setDateState({ day: null, month: null, year: null, date: null });
        }
    }, [date]);
    var handleDataChanged = React.useCallback(function (d) {
        var newDate = dateHelpers_1.DateHelpers.toDateFormat(d);
        var day = datePartOrder.indexOf("day") === -1 ? 1 : null;
        var year = datePartOrder.indexOf("year") === -1 ? 2000 : null;
        setDateState(tslib_1.__assign({}, d, { date: newDate, day: day, year: year }));
        if (onChange && newDate) {
            onChange(newDate);
        }
    }, [datePartOrder, onChange]);
    var validationMessage = formCore_1.DataValidationMessage.get(props);
    var options = React.useMemo(function () {
        return {
            day: options_1.buildOptions(dayLabel, daysArrayByMonth, function (v) { return v; }, function (v) { return formatting_1.Formatting.twoDigitNumber(parseInt(v, 10)); }),
            month: options_1.buildOptions(monthLabel, dateHelpers_1.DateHelpers.getMonthValues(), function (v) { return v.value; }, function (v) { return v.label; }),
            year: options_1.buildOptions(yearLabel, dateHelpers_1.DateHelpers.getYearValues(futureDates, yearsFromNow, startYearCap), function (v) { return v; }, function (v) { return v.toString(); }),
        };
    }, [dayLabel, monthLabel, yearLabel, daysArrayByMonth, futureDates, yearsFromNow, startYearCap]);
    var classes = React.useMemo(function () { return classHelpers_1.ClassHelpers.classNames("armstrong-input", "date-input", className, {
        "show-validation": (validationMode !== "none" && validationMessage),
        "input-disabled": disabled,
    }); }, [className, validationMessage, validationMode, disabled]);
    var _b = formHooks_1.useForm(dateState), DataForm = _b.DataForm, bind = _b.bind;
    return (React.createElement(DataForm, { className: classes, title: validationMessage, onDataChanged: handleDataChanged },
        React.createElement(grid_1.Grid, null,
            React.createElement(grid_1.Row, null, datePartOrder.map(function (key, idx) {
                return (React.createElement(grid_1.Col, { key: idx },
                    React.createElement("select", tslib_1.__assign({ tabIndex: tabIndex }, bind.selectNumeric(key), { disabled: disabled }), options[key])));
            })),
            React.createElement(validationWrapper_1.ValidationLabel, { message: validationMessage, mode: validationMode, wrapper: function (p) { return React.createElement(grid_1.Row, { height: "auto" },
                    React.createElement(grid_1.Col, tslib_1.__assign({}, p))); } }))));
};
exports.DateInput.defaultProps = {
    yearLabel: "Year",
    monthLabel: "Month",
    dayLabel: "Day",
    datePartOrder: ["day", "month", "year"],
    validationMode: "none",
};
